import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.openssl.PEMDecryptorProvider;
import org.bouncycastle.openssl.PEMEncryptedKeyPair;
import org.bouncycastle.openssl.PEMKeyPair;
import org.bouncycastle.openssl.PEMParser;
import org.bouncycastle.openssl.jcajce.JcaPEMKeyConverter;
import org.bouncycastle.openssl.jcajce.JcePEMDecryptorProviderBuilder;
import org.eclipse.paho.client.mqttv3.*;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileReader;
import java.security.KeyPair;
import java.security.KeyStore;
import java.security.Security;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.Date;

import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManagerFactory;

import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;

public class Publisher {

    public static final String BROKER_URL = "tcp://test.mosquitto.org";

    public static final String TOPIC = "conf";

    private MqttClient client;

    private Date date = new Date();

    private static String hurl;

    public Publisher(String hosturl) {
        if(hosturl != "") hurl = hosturl;
	else hurl = BROKER_URL;
	System.out.println("connecting to: "+hurl);

        //We have to generate a unique Client id.
        String clientId = "cloudappman-"+Long.toString(date.getTime()) + "-pub";


        try {

            client = new MqttClient(hurl, clientId,new MemoryPersistence());

        } catch (MqttException e) {
            e.printStackTrace();
            System.exit(1);
        }
    }

    public void start() {
    	String caFilePath = "";
		String clientCrtFilePath = "";
		String clientKeyFilePath = "";

        try {
            MqttConnectOptions options = new MqttConnectOptions();
            options.setCleanSession(false);

            if(hurl.contains("smartcity-challenge.edu-al.unipmn.it") || hurl.contains("193.206.52.98")) {  // devo connettermi al mosquitto in cloud
    			caFilePath = "./certificates/ca_certificate.pem";
    			clientCrtFilePath = "./certificates/client.crt";
    			clientKeyFilePath = "./certificates/client.key";

    			options.setUserName("pissir");
    			options.setPassword("pissir2020".toCharArray());
    		}

            options.setMqttVersion(MqttConnectOptions.MQTT_VERSION_3_1);

    		SSLSocketFactory socketFactory;
    		try {
    			socketFactory = getSocketFactory(caFilePath, clientCrtFilePath, clientKeyFilePath, "");
    			options.setSocketFactory(socketFactory);
    		} catch (Exception e) {
    			e.printStackTrace();
    		}

            client.connect(options);

        } catch (MqttException e) {
            e.printStackTrace();
            System.exit(1);
	}
//         catch (InterruptedException e) {
//            e.printStackTrace();
//        }
    }

    void publish(String topic, String msg) throws MqttException {
        MqttTopic pubTopic = client.getTopic(topic);

        pubTopic.publish(new MqttMessage(msg.getBytes()));

    }


    private static SSLSocketFactory getSocketFactory(final String caCrtFile,
			final String crtFile, final String keyFile, final String password)
			throws Exception {
		Security.addProvider(new BouncyCastleProvider());

		// load CA certificate
		X509Certificate caCert = null;

		FileInputStream fis = new FileInputStream(caCrtFile);
		BufferedInputStream bis = new BufferedInputStream(fis);
		CertificateFactory cf = CertificateFactory.getInstance("X.509");

		while (bis.available() > 0) {
			caCert = (X509Certificate) cf.generateCertificate(bis);
		}

		// load client certificate
		bis = new BufferedInputStream(new FileInputStream(crtFile));
		X509Certificate cert = null;
		while (bis.available() > 0) {
			cert = (X509Certificate) cf.generateCertificate(bis);
		}

		// load client private key
		PEMParser pemParser = new PEMParser(new FileReader(keyFile));
		Object object = pemParser.readObject();
		PEMDecryptorProvider decProv = new JcePEMDecryptorProviderBuilder()
				.build(password.toCharArray());
		JcaPEMKeyConverter converter = new JcaPEMKeyConverter()
				.setProvider("BC");
		KeyPair key;
		if (object instanceof PEMEncryptedKeyPair) {
			//System.out.println("Encrypted key - we will use provided password");
			key = converter.getKeyPair(((PEMEncryptedKeyPair) object)
					.decryptKeyPair(decProv));
		} else {
			//System.out.println("Unencrypted key - no password needed");
			key = converter.getKeyPair((PEMKeyPair) object);
		}
		pemParser.close();

		// CA certificate is used to authenticate server
		KeyStore caKs = KeyStore.getInstance(KeyStore.getDefaultType());
		caKs.load(null, null);
		caKs.setCertificateEntry("ca-certificate", caCert);
		TrustManagerFactory tmf = TrustManagerFactory.getInstance("X509");
		tmf.init(caKs);

		// client key and certificates are sent to server so it can authenticate
		// us
		KeyStore ks = KeyStore.getInstance(KeyStore.getDefaultType());
		ks.load(null, null);
		ks.setCertificateEntry("certificate", cert);
		ks.setKeyEntry("private-key", key.getPrivate(), password.toCharArray(),
				new java.security.cert.Certificate[] { cert });
		KeyManagerFactory kmf = KeyManagerFactory.getInstance(KeyManagerFactory
				.getDefaultAlgorithm());
		kmf.init(ks, password.toCharArray());

		// finally, create SSL socket factory
		SSLContext context = SSLContext.getInstance("TLSv1.2");
		context.init(kmf.getKeyManagers(), tmf.getTrustManagers(), null);

		return context.getSocketFactory();
	}
}
